library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity GH is

port( 
	CLK: in std_logic ;
	RST: in std_logic ;
	START : in std_logic;
	G: in std_logic ;
	C1: in std_logic ;
	C2: in std_logic ;
	N1: in std_logic ;
	N2: in std_logic ;
	Co1: out std_logic_vector ( 2 downto 0 );
	Co2: out std_logic_vector ( 2 downto 0 );
	Essais : out std_logic_vector ( 1 downto 0 ));

end;

architecture arch_GH of GH is

type ETAT2 is (Init, Play);
signal ETAT : ETAT2 := Init;

signal sCo1 : std_logic_vector ( 2 downto 0 ); -- FF corde 1, connects  Co1
signal sCo2 : std_logic_vector ( 2 downto 0 ); -- FF corde 2, connects  Co2
signal sEssais: std_logic_vector ( 1 downto 0 ); -- FF essais, connects  Essais

signal T1 : std_logic_vector(2 downto 0); -- signal = G C1 sCo1(2), bits  tester
signal T2 : std_logic_vector(2 downto 0); -- signal = G C2 sCo2(2), bits  tester

begin

-- *****************************************************
-- concatnation de std_logic pour simplifier la syntaxe
-- *****************************************************

-- a priori ne consomme rien !!
T1 <= G & C1 & sCo1(2);
T2 <= G & C2 & sCo2(2);

-- ***********************
-- process Shift Registers
-- ***********************

-- excut en mme temps que le process principal !!
--process(CLK)
--begin
--  if rising_edge(CLK) then
--    if RST='1' then
--      sCo1    <= "000";
--      sCo2    <= "000";
--    else
--      if ETAT = Play then -- peut-tre test mais non affect dans ce process !
--	-- shift registers des cordes
--	sCo1 <= sCo1(1 downto 0) & N1;
--	sCo2 <= sCo2(1 downto 0) & N2;
--      end if;
--    end if;
--  end if;
--end process;

-- *****************
-- process principal
-- *****************

-- excut en parallle au process Shift Register
-- donc les valeurs testes du registres ne sont
-- pas encore modifies
-- si cette manire de programmer vous pose des problmes 
-- alors, commentez le process Shift Registers
-- et dcommentez les lignes avec sCo1 et sCo2 ci-dessous;

process(CLK)
begin
  if rising_edge(CLK) then 	-- front montant de CLK comme spcifi dans datasheet
    if RST='1' then 		-- reset
      sEssais <= "11";
      ETAT    <= Init;
      sCo1    <= "000";	-- si on supprime le process Shift Registers
      sCo2    <= "000"; 
    else
      case ETAT is
	when Init =>		-- attendre l'appui du start
          if START = '1' then ETAT <= Play;
          end if;
        when Play =>		-- on joue
 	  if ((T1="100" and T2="100") or 
		(T1="111" and T2="100") or
		(T1="100" and T2="111") or
		(T1="111" and T2="111")) then
	    ETAT <= Play;	-- reussi => on ne change rien
   	  else			-- rat => on dcrmente le nombre d'essais
  	    if sEssais = "00" then
	      ETAT <= Init;
            else
	      sEssais <= sEssais - 1;
	    end if;
     	  end if;
        -- pas besoin de "when others" car toutes le valeurs possibles ont t considres
        -- par contre, si c'tait un vecteur std_logic_vector, alors le 
	-- "when others" doit tre prsent

	  -- shift registers des cordes	-- si on supprime le process Shift Registers
	  sCo1 <= sCo1(1 downto 0) & N1;
	  sCo2 <= sCo2(1 downto 0) & N2;
      end case;
    end if; 			-- end reset
  end if; 			-- end rising_edge
end process;

-- *********************
-- sorties combinatoires
-- *********************

-- ici, on fait le lien entre la sortie du FF 
-- et la pin du composant

Co1 <= sCo1;
Co2 <= sCo2;
Essais <= sEssais;

end arch_GH;
